@@ -6,25 +6,22 @@ Created on 2015-8-14
 """
 
 from PyFin.DateUtilities import Date
-from PyFin.Patterns.Singleton import Singleton
 from PyFin.Utilities import pyFinAssert
 
 
-class SettingsFactory(Singleton):
-    def __init__(self, forcedBuild=False):
+class SettingsFactory(object):
+    def __init__(self):
         self._evaluationDate = None
-        self._defaultSymbolList = set(['600000.xshg'])
 
     @property
     def evaluationDate(self):
-        if self._evaluationDate is None:
+        if not self._evaluationDate:
             return Date.todaysDate()
         return self._evaluationDate
 
     @evaluationDate.setter
     def evaluationDate(self, value):
-        if not isinstance(value, Date):
-            raise ValueError("{0} is not a valid PyFin date object".format(value))
+        pyFinAssert(isinstance(value, Date), ValueError, "{0} is not a valid PyFin date object".format(value))
         self._evaluationDate = value
 
     def resetEvaluationDate(self):
@@ -34,14 +31,5 @@ class SettingsFactory(Singleton):
         if self._evaluationDate is None:
             self._evaluationDate = Date.todaysDate()
 
-    @property
-    def defaultSymbolList(self):
-        return self._defaultSymbolList
-
-    @defaultSymbolList.setter
-    def defaultSymbolList(self, value):
-        pyFinAssert(len(value) != 0, ValueError, "default symbol list can't be set to empty")
-        self._defaultSymbolList = [v.lower() for v in value]
-
 
 Settings = SettingsFactory()
