@@ -11,7 +11,6 @@ from PyFin.DateUtilities import Period
 from PyFin.DateUtilities import Schedule
 from PyFin.Enums import BizDayConventions
 from PyFin.DateUtilities import check_date
-from PyFin.Utilities.Asserts import pyFinAssert
 
 
 def isBizDay(holidayCenter, ref):
@@ -23,7 +22,6 @@ def isBizDay(holidayCenter, ref):
 def datesList(fromDate, toDate):
     fromDate = check_date(fromDate)
     toDate = check_date(toDate)
-    pyFinAssert(fromDate <= toDate, ValueError, "from date ({0} must be earlier than to date {1}".format(fromDate, toDate))
     return [Date.fromExcelSerialNumber(serial).toDateTime() for serial in
             range(fromDate.serialNumber, toDate.serialNumber + 1)]
 
