@@ -17,6 +17,7 @@ from PyFin.api.DateUtilities import nthWeekDay
 from PyFin.api.DateUtilities import makeSchedule
 
 from PyFin.Analysis import transform
+from PyFin.api.Analysis import SeriesValues
 from PyFin.api.Analysis import SIGN
 from PyFin.api.Analysis import EMA
 from PyFin.api.Analysis import MACD
@@ -72,6 +73,7 @@ __all__ = ["datesList",
            "makeSchedule",
            "transform",
            "SIGN",
+           "SeriesValues",
            "EMA",
            "MACD",
            "RSI",
